unit fmChartTest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeeGDIPlus, TeEngine, ExtCtrls, TeeProcs, Chart, TeeSurfa, TeeMapSeries,
  TeCanvas, Math;

type
  TForm31 = class(TForm)
    Chart1: TChart;
    Chart2: TChart;
    TeeGDIPlus2: TTeeGDIPlus;
    Series1: TMapSeries;
    Series2: TMapSeries;
    procedure FormCreate(Sender: TObject);
    procedure Chart1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Chart2MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Chart2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure Chart1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Chart2MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    MouseDownChart1X: integer;
    MouseDownChart1Y: integer;
    MouseDownChart2X: integer;
    MouseDownChart2Y: integer;
  public
    { Public declarations }
    function ShapeInRect( const aShape: TTeePolygon;
                          const aRect: TRect ): boolean;
    function PointInShape( const aShape: TTeePolygon;
                           const aClicked: TPoint ): boolean;
  end;

var
  Form31: TForm31;

implementation

{$R *.dfm}

procedure TForm31.Chart1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseDownChart1X:=X;
  MouseDownChart1Y:=Y;
end;

procedure TForm31.Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  lRect: TRect;
  lShape: TTeePolygon;
  i: Integer;
begin
  if MouseDownChart1X >= 0 then
  begin
    lRect:=Rect(Min(MouseDownChart1X, X), Min(MouseDownChart1Y, Y), Max(MouseDownChart1X, X), Max(MouseDownChart1Y, Y));
    for i:=0 to Series1.Count-1 do
    begin
      lShape := Series1.Shapes[ i ];
      if (ShapeInRect( lShape, lRect )) then
      begin
        lShape.ParentPen := False;
        lShape.ParentBrush := False;
        lShape.Brush.Style := bsSolid;
        lShape.Pen.Style := psSolid;
        lShape.Pen.Width := 2;
        lShape.Gradient.Visible := True;
        lShape.Gradient.StartColor := clWhite;
        lShape.Gradient.EndColor := $00A000;
      end
      else
      begin
        lShape.ParentPen := True;
        lShape.ParentBrush := True;
        lShape.Pen.Color := clBlack;
        lShape.Gradient.Visible := True;
        lShape.Gradient.StartColor := clWhite;
        lShape.Gradient.EndColor := $004000;
      end;

    end;

    Chart1.Repaint;
    Chart1.Canvas.Brush.Style:=bsClear;
    Chart1.Canvas.Pen.Color:=clBlack;
    Chart1.Canvas.Rectangle(lRect);
  end;
end;

procedure TForm31.Chart1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseDownChart1X:=-1;
  MouseDownChart1X:=-1;
  Chart1.Repaint;
end;

procedure TForm31.Chart2MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseDownChart2X:=X;
  MouseDownChart2Y:=Y;
end;

procedure TForm31.Chart2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  lRect: TRect;
  lShape: TTeePolygon;
  i: Integer;
begin
  if MouseDownChart2X >= 0 then
  begin
    lRect:=Rect(Min(MouseDownChart2X, X), Min(MouseDownChart2Y, Y), Max(MouseDownChart2X, X), Max(MouseDownChart2Y, Y));
    for i:=0 to Series2.Count-1 do
    begin
      lShape := Series2.Shapes[ i ];
      if (ShapeInRect( lShape, lRect )) then
      begin
        lShape.ParentPen := False;
        lShape.ParentBrush := False;
  //      lShape.Color := $00A000;
        lShape.Brush.Style := bsSolid;
        lShape.Pen.Style := psSolid;
        lShape.Pen.Width := 2;
        lShape.Gradient.Visible := True;
        lShape.Gradient.StartColor := clWhite;
        lShape.Gradient.EndColor := $00A000;
      end
      else
      begin
        lShape.ParentPen := True;
        lShape.ParentBrush := True;
  //      lShape.Color := $008000;
        lShape.Pen.Color := clBlack;
        lShape.Gradient.Visible := True;
        lShape.Gradient.StartColor := clWhite;
        lShape.Gradient.EndColor := $002000;
      end;

    end;

    Chart2.Repaint;
    Chart2.Canvas.Brush.Style:=bsClear;
    Chart2.Canvas.Pen.Color:=clBlack;
    Chart2.Canvas.Rectangle(lRect);
  end;
end;

procedure TForm31.Chart2MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseDownChart2X:=-1;
  MouseDownChart2X:=-1;
  Chart2.Repaint;
end;

procedure TForm31.FormCreate(Sender: TObject);
begin
  MouseDownChart1X:=-1;
  MouseDownChart1Y:=-1;
  MouseDownChart2X:=-1;
  MouseDownChart2Y:=-1;

  Chart1.View3D := False;
  Chart1.Legend.Visible:=false;
  Chart1.AllowPanning:=pmNone;
  Chart1.AllowZoom:=False;
  Series1.FillSampleValues( 12 );

  Chart2.View3D := False;
  Chart2.Legend.Visible:=false;
  Chart2.AllowPanning:=pmNone;
  Chart2.AllowZoom:=False;
  Series2.FillSampleValues( 12 );

end;

function TForm31.PointInShape(const aShape: TTeePolygon; const aClicked: TPoint): boolean;
var
  i, j: integer;
  lPolyPoint: TPoint;
  lPolyPointPrev: TPoint;
  lPointArray: TPointArray;
begin
  Result := False;
  lPointArray := aShape.GetPoints;
  j := aShape.Points.Count-1;

  for i := 0 to aShape.Points.Count-1 do
  begin
    lPolyPoint := lPointArray[ i ];
    lPolyPointPrev := lPointArray[ j ];
    if ( (     ( lPolyPoint.y < aClicked.Y )
           and ( lPolyPointPrev.Y >= aClicked.Y )
            or ( lPolyPointPrev.Y < aClicked.Y )
           and ( lPolyPoint.Y >= aClicked.Y )
         ) and
         (     ( lPolyPoint.X <= aClicked.X )
            or ( lPolyPointPrev.X <= aClicked.X ) )
       ) then
    begin
      if (   ( lPolyPoint.X + ( aClicked.Y - lPolyPoint.Y )
           / ( lPolyPointPrev.Y - lPolyPoint.Y )
           * ( lPolyPointPrev.X-lPolyPoint.x ) ) < aClicked.X ) then
      begin
        Result := not Result;
      end;
    end;
    j := i;
  end;
end;


function TForm31.ShapeInRect(const aShape: TTeePolygon; const aRect: TRect): boolean;
var
  i: integer;
  lPointArray: TPointArray;
begin
  Result:=False;
  lPointArray := aShape.Points.Polygon.GetPoints;
  for i:=0 to aShape.Points.Count-1 do
  begin
    if ( PointInRect( aRect, lPointArray[i] ) ) then
    begin
      Result:=True;
      break;
    end;
  end;
end;

end.
